
#ifndef __MVTITLE_H_
#define __MVTITLE_H_

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************
*
* MVTtl13.H
*
* Copyright (c) Microsoft Corporation 1993
* All rights reserved.
*
*************************************************************************
*
*   The MVTitle API
*
* Module intent: Header file for the MVTTL13.DLL.
*
************************************************************************/

typedef struct _TITLE TITLE, NEAR *PTITLE, FAR *LPTITLE;

#pragma pack(4)

#ifndef TITLE_DEFINED
#define TITLE_DEFINED

typedef HANDLE HTITLE;    // Handle to a Title
typedef HANDLE HTLIST;    // handle to a Topic List
typedef HANDLE HWHEEL;
typedef struct
  {
  int iError;
  int iStart;
  int iLength;
  } QUERYERR, FAR * LPQUERYERR;

#define IMPLICIT_AND    0
#define IMPLICIT_OR     1

#define TL_AND  0
#define TL_OR   1
#define TL_NOT  2

// These must be the same as equivalents in FONT.H!
#define fSMALLCAPS     0x0004
#define fNOUNDERLINE   0x0008
#define fDOTUNDERLINE  0x0010
#define fWORDUNDERLINE 0x0020
#define fDBLUNDERLINE  0x0040

#define STYLESIZE      64


#endif

// define FILETIME for Win16
#if !defined(_WIN32) && !defined(_FILETIME_)
#define _FILETIME_
typedef struct _FILETIME {
    DWORD dwLowDateTime;
    DWORD dwHighDateTime;
} FILETIME, *PFILETIME, FAR *LPFILETIME;

#endif

#pragma pack()

//
// Title API
//
HTITLE FAR PASCAL EXPORT_API TitleOpenEx(LPCSTR, FARPROC, LPVOID, BOOL);
void   FAR PASCAL EXPORT_API TitleClose(HTITLE);
long   FAR PASCAL EXPORT_API TitleGetInfo(HTITLE, int, long, long);
BOOL   FAR PASCAL EXPORT_API TitleValid(HTITLE);
HANDLE FAR PASCAL EXPORT_API TitleQuery(HTITLE,WORD,LPCSTR,LPVOID,LPCSTR,WORD,
											LPVOID,DWORD	FAR *,DWORD FAR *,LPQUERYERR);

BOOL FAR PASCAL EXPORT_API TitleTime(HTITLE, FILETIME FAR *);
HANDLE FAR PASCAL EXPORT_API TitleLoadDLL (HTITLE hTitle, LPCSTR lpszName, LPWORD lpErr);


//
// Addressing API
//
 ADDR   FAR PASCAL EXPORT_API addrConvertHash(HTITLE, HASH);
 ADDR   FAR PASCAL EXPORT_API addrConvertTopicNumber(HTITLE, LONG);
 ADDR   FAR PASCAL EXPORT_API addrConvertContextString(HTITLE, LPCSTR);
 ADDR   FAR PASCAL EXPORT_API addrGetContents(HTITLE);
 DWORD  FAR PASCAL EXPORT_API vaConvertAddr(HTITLE, ADDR);
 DWORD  FAR PASCAL EXPORT_API vaConvertHash(HTITLE, HASH);
 DWORD  FAR PASCAL EXPORT_API vaConvertTopicNumber(HTITLE, LONG);
 DWORD  FAR PASCAL EXPORT_API vaConvertContextString(HTITLE, LPCSTR);
 DWORD  FAR PASCAL EXPORT_API vaGetContents(HTITLE);


//
// Searching API
//
// these take a handle to the SearchHighlights, returned by TopicListFromQuery
HANDLE FAR PASCAL EXPORT_API HighlightsInTopic(HANDLE hGlobalHighlights, long l);
ADDR FAR PASCAL EXPORT_API addrSearchHighlight(HANDLE hGlobalHighlights, long lIndex, long l);
LONG FAR PASCAL EXPORT_API SearchHighlightCount(HANDLE hGlobalHighlights, long lIndex, LPWORD lpErr);
LONG EXPORT_API FAR PASCAL HighlightLookup(HANDLE hGlobalHighlights, long l) ;
void EXPORT_API FAR PASCAL HighlightDestroy(HANDLE hGlobalHighlight);
int FAR PASCAL EXPORT_API HighlightGetGroup(HANDLE hGlobalHighlights, LPVOID lpGroup);


//
//  Word Wheel API.
//
ERR FAR PASCAL EXPORT_API WordWheelQuery(HANDLE hWheel,WORD wFlags,LPCSTR lpstrQuery,LPVOID lpGroupHits);
HWHEEL FAR PASCAL EXPORT_API WordWheelOpenTitle(HANDLE hTitle, LPCSTR lpstr, BOOL fReadOnly);
void   FAR PASCAL EXPORT_API WordWheelClose(HWHEEL);
long   FAR PASCAL EXPORT_API WordWheelLength(HWHEEL);
long   FAR PASCAL EXPORT_API WordWheelPrefix(HWHEEL,LPCSTR);
BOOL   FAR PASCAL EXPORT_API WordWheelLookup(HWHEEL,long,LPBYTE,int);
long   FAR PASCAL EXPORT_API KeyIndexGetAddrs (HWHEEL,LPBYTE,int,int,LPBYTE);
WORD   FAR PASCAL EXPORT_API KeyIndexGetCount (HWHEEL,LPBYTE);

//
// Baggage File API
//
VOID EXPORT_API FAR PASCAL BaggageClose (HANDLE hFile, LPERR lpErr);
LONG EXPORT_API FAR PASCAL BaggageSeek (HANDLE hFile, LONG lOffset, int nOrigin, LPERR lpErr);
LONG EXPORT_API FAR PASCAL BaggageRead (HANDLE hFile, BYTE FAR* lpBuf, DWORD dwBytes, LPERR lpErr);
HANDLE EXPORT_API FAR PASCAL BaggageOpen (HTITLE hTitle, LPCSTR lpszFilename, BOOL fBaggage, LPERR lpErr);
LONG EXPORT_API FAR PASCAL BaggageSize (HANDLE hFile, LPERR lpErr);
LONG EXPORT_API FAR PASCAL BaggageSeekRead (HANDLE h, LONG lOffset, int nOrigin, 
											LPBYTE lpBuf, DWORD dwBytes, LPERR lpErr);
HANDLE EXPORT_API FAR PASCAL BaggageGetFile (HTITLE hTitle, LPCSTR lpszFilename, BOOL fBaggage,
											LONG lOffset, DWORD dwBytes,  DWORD FAR *lpdwRead, LPERR lpErr);

#ifdef __cplusplus
}
#endif

#endif  // __MVTITLE_H_
 
